@@ -43,28 +43,28 @@ nftnl_expr_payload_set(struct nftnl_expr *e, uint16_t type,
 
 	switch(type) {
 	case NFTNL_EXPR_PAYLOAD_SREG:
-		payload->sreg = *((uint32_t *)data);
+		memcpy(&payload->sreg, data, sizeof(payload->sreg));
 		break;
 	case NFTNL_EXPR_PAYLOAD_DREG:
-		payload->dreg = *((uint32_t *)data);
+		memcpy(&payload->dreg, data, sizeof(payload->dreg));
 		break;
 	case NFTNL_EXPR_PAYLOAD_BASE:
-		payload->base = *((uint32_t *)data);
+		memcpy(&payload->base, data, sizeof(payload->base));
 		break;
 	case NFTNL_EXPR_PAYLOAD_OFFSET:
-		payload->offset = *((unsigned int *)data);
+		memcpy(&payload->offset, data, sizeof(payload->offset));
 		break;
 	case NFTNL_EXPR_PAYLOAD_LEN:
-		payload->len = *((unsigned int *)data);
+		memcpy(&payload->len, data, sizeof(payload->len));
 		break;
 	case NFTNL_EXPR_PAYLOAD_CSUM_TYPE:
-		payload->csum_type = *((uint32_t *)data);
+		memcpy(&payload->csum_type, data, sizeof(payload->csum_type));
 		break;
 	case NFTNL_EXPR_PAYLOAD_CSUM_OFFSET:
-		payload->csum_offset = *((uint32_t *)data);
+		memcpy(&payload->csum_offset, data, sizeof(payload->csum_offset));
 		break;
 	case NFTNL_EXPR_PAYLOAD_FLAGS:
-		payload->csum_flags = *((uint32_t *)data);
+		memcpy(&payload->csum_flags, data, sizeof(payload->csum_flags));
 		break;
 	default:
 		return -1;
@@ -231,59 +231,6 @@ static inline int nftnl_str2base(const char *base)
 	}
 }
 
-static int
-nftnl_expr_payload_json_parse(struct nftnl_expr *e, json_t *root,
-				 struct nftnl_parse_err *err)
-{
-#ifdef JSON_PARSING
-	const char *base_str;
-	uint32_t reg, uval32;
-	int base;
-
-	if (nftnl_jansson_parse_reg(root, "dreg", NFTNL_TYPE_U32, &reg, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_PAYLOAD_DREG, reg);
-
-	base_str = nftnl_jansson_parse_str(root, "base", err);
-	if (base_str != NULL) {
-		base = nftnl_str2base(base_str);
-		if (base < 0)
-			return -1;
-
-		nftnl_expr_set_u32(e, NFTNL_EXPR_PAYLOAD_BASE, base);
-	}
-
-	if (nftnl_jansson_parse_val(root, "offset", NFTNL_TYPE_U32, &uval32,
-				  err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_PAYLOAD_OFFSET, uval32);
-
-	if (nftnl_jansson_parse_val(root, "len", NFTNL_TYPE_U32, &uval32, err) == 0)
-		nftnl_expr_set_u32(e, NFTNL_EXPR_PAYLOAD_LEN, uval32);
-
-	return 0;
-#else
-	errno = EOPNOTSUPP;
-	return -1;
-#endif
-}
-
-static int nftnl_expr_payload_export(char *buf, size_t size, uint32_t flags,
-				     const struct nftnl_expr *e, int type)
-{
-	struct nftnl_expr_payload *payload = nftnl_expr_data(e);
-	NFTNL_BUF_INIT(b, buf, size);
-
-	if (e->flags & (1 << NFTNL_EXPR_PAYLOAD_DREG))
-		nftnl_buf_u32(&b, type, payload->dreg, DREG);
-	if (e->flags & (1 << NFTNL_EXPR_PAYLOAD_OFFSET))
-		nftnl_buf_u32(&b, type, payload->offset, OFFSET);
-	if (e->flags & (1 << NFTNL_EXPR_PAYLOAD_LEN))
-		nftnl_buf_u32(&b, type, payload->len, LEN);
-	if (e->flags & (1 << NFTNL_EXPR_PAYLOAD_BASE))
-		nftnl_buf_str(&b, type, base2str(payload->base), BASE);
-
-	return nftnl_buf_done(&b);
-}
-
 static int
 nftnl_expr_payload_snprintf(char *buf, size_t len, uint32_t type,
 			    uint32_t flags, const struct nftnl_expr *e)
@@ -305,49 +252,19 @@ nftnl_expr_payload_snprintf(char *buf, size_t len, uint32_t type,
 					payload->offset, payload->dreg);
 	case NFTNL_OUTPUT_XML:
 	case NFTNL_OUTPUT_JSON:
-		return nftnl_expr_payload_export(buf, len, flags, e, type);
 	default:
 		break;
 	}
 	return -1;
 }
 
-static bool nftnl_expr_payload_cmp(const struct nftnl_expr *e1,
-				   const struct nftnl_expr *e2)
-{
-	struct nftnl_expr_payload *p1 = nftnl_expr_data(e1);
-	struct nftnl_expr_payload *p2 = nftnl_expr_data(e2);
-	bool eq = true;
-
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_SREG))
-		eq &= (p1->sreg == p2->sreg);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_DREG))
-		eq &= (p1->dreg == p2->dreg);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_BASE))
-		eq &= (p1->base == p2->base);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_OFFSET))
-		eq &= (p1->offset == p2->offset);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_LEN))
-		eq &= (p1->len == p2->len);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_CSUM_TYPE))
-		eq &= (p1->csum_type == p2->csum_type);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_CSUM_OFFSET))
-		eq &= (p1->csum_offset == p2->csum_offset);
-	if (e1->flags & (1 << NFTNL_EXPR_PAYLOAD_FLAGS))
-		eq &= (p1->csum_flags == p2->csum_flags);
-
-	return eq;
-}
-
 struct expr_ops expr_ops_payload = {
 	.name		= "payload",
 	.alloc_len	= sizeof(struct nftnl_expr_payload),
 	.max_attr	= NFTA_PAYLOAD_MAX,
-	.cmp		= nftnl_expr_payload_cmp,
 	.set		= nftnl_expr_payload_set,
 	.get		= nftnl_expr_payload_get,
 	.parse		= nftnl_expr_payload_parse,
 	.build		= nftnl_expr_payload_build,
 	.snprintf	= nftnl_expr_payload_snprintf,
-	.json_parse	= nftnl_expr_payload_json_parse,
 };
